#include "owl.h"

void owl_process_key(int j, int meta) {
  WINDOW *typwin;
  owl_editwin *tw;
  
  typwin=owl_global_get_curs_typwin(&g);
  tw=owl_global_get_typwin(&g);
  
  if (j==ERR) return;

  /* first, keys that are not context sensitive */
  if (j==26) {
    /* C-z will suspend */
    owl_command_suspend(&g);
    return;
  } else if (j==12) {
    /* C-l will refresh */
    owl_function_refresh();
    owl_global_set_needrefresh(&g);
  }

  /* if the typwin is not active, use the main keybindings */
  if (!owl_global_is_typwin_active(&g)) {
    owl_function_makemsg("");
    if (!meta) {
      if (j==' ') {
	owl_global_set_needrefresh(&g);
	owl_mainwin_redisplay(owl_global_get_mainwin(&g));
      } else if (j=='A') {
	if (!owl_global_is_zaway(&g)) {
	  owl_function_zaway_on(NULL);
	} else {
	  owl_function_zaway_off();
	}
      } else if (j=='d') {
	owl_command_delete_cur();
      } else if (j=='u') {
	owl_command_undelete_curmsg();
      } else if (j=='x') {
	owl_command_expunge();
      } else if (j=='i') {
	owl_command_info();
      } else if (j=='h') {
	owl_command_help();
      } else if (j=='n') {
	owl_command_next_notdeleted();
      } else if (j=='p') {
	owl_command_prev_notdeleted();
      } else if (j==KEY_DOWN || j==14) {
	owl_command_next();
      } else if (j==KEY_UP || j==16) {
	owl_command_prev();
      } else if (j==KEY_RIGHT) {
	owl_command_shift_right();
      } else if (j==KEY_LEFT) {
	owl_command_shift_left();
      } else if (j=='<') {
	owl_global_set_rightshift(&g, 0);
	owl_command_first();
      } else if (j=='>') {
	owl_command_last();
      } else if (j=='w') {
	owl_function_openurl();
      } else if (j=='z') {
	/* this is ugly, do better */
	int i;
	owl_global_set_typwin_active(&g);
	owl_editwin_set_locktext(tw, "command: ");
	owl_global_set_needrefresh(&g);
	
	for (i=0; i<7; i++) {
	  owl_editwin_process_char(tw, "zwrite "[i], 0);
	}
	wrefresh(typwin);
	return;
      } else if (j=='r') {
	owl_process_command("reply");
	return;
      } else if (j==':') {
	owl_global_set_typwin_active(&g);
	owl_editwin_set_locktext(tw, "command: ");
	owl_global_set_needrefresh(&g);
	wrefresh(typwin);
	return;
      } else {
	return;
      }
    } else {
      if (j=='d') {
	owl_function_makemsg("Got meta d");
      } else if (j=='<') {
	owl_global_set_rightshift(&g, 0);
	owl_command_first();
      } else if (j=='>') {
	owl_command_last();
      } else if (j=='x') {
	/* same as ':' above */
	owl_global_set_typwin_active(&g);
	owl_editwin_set_locktext(tw, "command: ");
	owl_global_set_needrefresh(&g);
	wrefresh(typwin);
	return;
      }
    }
    sepbar(NULL);
    return;
  }

  /* otherwise keys go to the typwin */
  owl_editwin_process_char(tw, j, meta);
  
  /* C-c will cancel */
  if (j == 3) {
    owl_function_makemsg("Command cancelled.");
    owl_editwin_fullclear(tw);
    wrefresh(typwin);
    owl_global_set_typwin_inactive(&g);
    owl_editwin_new_style(tw, OWL_EDITWIN_STYLE_ONELINE);
    return;
  }
  
  /* C-d will do whatever command if in MULTILINE mode */
  if ((owl_editwin_get_style(tw)==OWL_EDITWIN_STYLE_MULTILINE) &&
      (j==4)) {
    owl_function_run_buffercommand();
    owl_editwin_new_style(tw, OWL_EDITWIN_STYLE_ONELINE);
    owl_editwin_fullclear(tw);
    owl_global_set_typwin_inactive(&g);
    wrefresh(typwin);
    return;
  }
  
  /* if we're in ONELINE mode and they hit enter, process the line */
  if ((owl_editwin_get_style(tw)==OWL_EDITWIN_STYLE_ONELINE) &&
      (j=='\n' || j=='\r')) {
    
    owl_global_set_typwin_inactive(&g);
    owl_process_command(owl_editwin_get_text(tw));
    
    /* if we're still in ONELINE mode we can clear the buffer */
    if (owl_editwin_get_style(tw)==OWL_EDITWIN_STYLE_ONELINE) {
      owl_editwin_fullclear(tw);
    }
    wrefresh(typwin);
  }
}
